import { BehaviorSubject } from 'rxjs'
import * as sourcegraph from 'sourcegraph'

/**
 * calls next() on behaviorSubject with a immutably added element ([...old, value])
 *
 * @param behaviorSubject subject that holds a collection
 * @param value to add to a collection
 * @returns Unsubscribable that will remove that element from the behaviorSubject.value and call next() again
 */
export function addWithRollback<T>(
    behaviorSubject: BehaviorSubject<readonly T[]>,
    value: T
): sourcegraph.Unsubscribable {
    behaviorSubject.next([...behaviorSubject.value, value])
    return {
        unsubscribe: () => behaviorSubject.next(behaviorSubject.value.filter(item => item !== value)),
    }
}
