package main

// Location specifies the first position in a source range.
type Location struct {
	Repo      string
	Rev       string
	Path      string
	Line      int
	Character int
}

// TEST DATA NOTES
//
// Zap release tags:
//   -`v1.14.1` at commit `2aa9fa2` and
//   - v1.12.0` at commit `a6015e1`.
//
// Dependencies:
//   - TiDB depends on zap v1.14.1
//   - etcd depends on zap v1.14.1
//   - titan depends on zap v1.12.0.
//
// All the test cases find definitions and references for the
// symbol `zap.String`, a global function defined in fields.go
// in both versions.

var testCases = []struct {
	Definition Location   // Symbol definition
	References []Location // All xrepo references of definition in test repository set
}{
	{
		Definition: Location{
			Repo:      "github.com/sourcegraph-testing/zap",
			Rev:       "2aa9fa25da83bdfff756c36a91442edc9a84576c",
			Path:      "field.go",
			Line:      215,
			Character: 5,
		},
		References: []Location{
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/jwt.go", Line: 64, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/jwt.go", Line: 72, Character: 37},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/jwt.go", Line: 100, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/jwt.go", Line: 109, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/jwt.go", Line: 111, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/range_perm_cache.go", Line: 89, Character: 36},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/range_perm_cache.go", Line: 102, Character: 36},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/range_perm_cache.go", Line: 115, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/simple_token.go", Line: 131, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/simple_token.go", Line: 132, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/simple_token.go", Line: 159, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/simple_token.go", Line: 160, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 311, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 312, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 346, Character: 37},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 387, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 421, Character: 32},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 427, Character: 47},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 450, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 463, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 495, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 522, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 524, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 541, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 543, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 580, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 581, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 620, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 623, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 691, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 692, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 693, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 700, Character: 47},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 741, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 768, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 830, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 831, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 862, Character: 61},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 925, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 999, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1132, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1133, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1140, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1141, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1167, Character: 39},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1188, Character: 41},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1197, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1198, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1238, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1295, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "auth/store.go", Line: 1296, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 61, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 62, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 98, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 99, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 100, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 144, Character: 58},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 148, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 149, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 162, Character: 40},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 162, Character: 82},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 162, Character: 132},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 165, Character: 37},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 181, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 182, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 183, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 184, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 204, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 205, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 206, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 208, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 215, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 216, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 218, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 220, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 221, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 222, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 257, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 258, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 259, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 279, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 280, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/balancer.go", Line: 281, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/connectivity/connectivity.go", Line: 80, Character: 66},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/picker/roundrobin_balanced.go", Line: 71, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/picker/roundrobin_balanced.go", Line: 72, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/picker/roundrobin_balanced.go", Line: 81, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/balancer/picker/roundrobin_balanced.go", Line: 82, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/retry_interceptor.go", Line: 54, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/retry_interceptor.go", Line: 63, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/retry_interceptor.go", Line: 80, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/retry_interceptor.go", Line: 290, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 112, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 123, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 136, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 143, Character: 24},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 289, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 290, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 291, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 292, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 302, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 303, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 304, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "clientv3/snapshot/v3_snapshot.go", Line: 305, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 610, Character: 57},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 653, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 654, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 655, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 656, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 668, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 669, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 670, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config.go", Line: 671, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config_logging.go", Line: 178, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config_logging.go", Line: 179, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config_logging.go", Line: 187, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/config_logging.go", Line: 188, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 233, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 264, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 265, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 266, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 267, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 268, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 272, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 273, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 274, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 275, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 276, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 278, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 279, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 290, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 291, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 292, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 296, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 297, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 299, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 300, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 301, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 315, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 316, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 420, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 434, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 447, Character: 108},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 450, Character: 123},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 494, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 499, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 511, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 527, Character: 42},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 535, Character: 108},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 538, Character: 118},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 592, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 615, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/etcd.go", Line: 681, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/serve.go", Line: 140, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/serve.go", Line: 188, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/serve.go", Line: 250, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/serve.go", Line: 314, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "embed/serve.go", Line: 401, Character: 65},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/config.go", Line: 299, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/config.go", Line: 323, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/config.go", Line: 324, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 79, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 90, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 101, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 102, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 112, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 123, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 143, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 144, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 149, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 157, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 166, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 283, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 289, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 301, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 357, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 407, Character: 64},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 425, Character: 48},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 438, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/etcd.go", Line: 439, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 194, Character: 39},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 200, Character: 59},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 227, Character: 35},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 299, Character: 39},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 329, Character: 57},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 413, Character: 42},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/grpc_proxy.go", Line: 480, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/util.go", Line: 40, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/util.go", Line: 57, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/util.go", Line: 67, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdmain/util.go", Line: 76, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/capability.go", Line: 75, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 119, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 120, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 133, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 134, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 142, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 143, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 153, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/base.go", Line: 154, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/metrics.go", Line: 59, Character: 32},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/metrics.go", Line: 111, Character: 56},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/etcdhttp/peer.go", Line: 147, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 163, Character: 54},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 271, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 272, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 273, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 280, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 365, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 387, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 388, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 389, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 413, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 414, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 415, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 421, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 422, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 423, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 447, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 448, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 449, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 455, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 456, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 457, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 476, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 477, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 495, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 496, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 497, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 517, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 518, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 519, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 520, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 525, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 526, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 527, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 570, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 571, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 600, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 601, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 626, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 627, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 643, Character: 51},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 661, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 679, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 730, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 777, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 778, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 791, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 792, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 822, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/cluster.go", Line: 823, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/store.go", Line: 99, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/store.go", Line: 109, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/store.go", Line: 116, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/store.go", Line: 131, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/store.go", Line: 146, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/membership/store.go", Line: 156, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 114, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 126, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 143, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 223, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 224, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 239, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 240, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 241, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 255, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 256, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 259, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 268, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 269, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 282, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 283, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 286, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 299, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 300, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 365, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 366, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 367, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 378, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 379, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 380, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 389, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 390, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 391, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 408, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 409, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 410, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 411, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 421, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 422, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 423, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 424, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 425, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 481, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 482, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 483, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 484, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 485, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 486, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 487, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 495, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 496, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 497, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 498, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 499, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 500, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 501, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/http.go", Line: 502, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 127, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 131, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 251, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 252, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 253, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 254, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 255, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 263, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 264, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 265, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 266, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 267, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 293, Character: 41},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 324, Character: 40},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer.go", Line: 329, Character: 40},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer_status.go", Line: 52, Character: 38},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer_status.go", Line: 65, Character: 70},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/peer_status.go", Line: 75, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/pipeline.go", Line: 72, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/pipeline.go", Line: 73, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/pipeline.go", Line: 85, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/pipeline.go", Line: 86, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/probing_status.go", Line: 51, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/probing_status.go", Line: 69, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/probing_status.go", Line: 70, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/probing_status.go", Line: 83, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/probing_status.go", Line: 84, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 61, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 62, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 63, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 64, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 72, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 73, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 74, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/remote.go", Line: 75, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/snapshot_sender.go", Line: 84, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/snapshot_sender.go", Line: 86, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/snapshot_sender.go", Line: 102, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/snapshot_sender.go", Line: 104, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/snapshot_sender.go", Line: 133, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/snapshot_sender.go", Line: 135, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 74, Character: 41},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 170, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 171, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 195, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 196, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 197, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 224, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 225, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 226, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 244, Character: 46},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 250, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 251, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 252, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 266, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 267, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 268, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 275, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 276, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 277, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 287, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 288, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 295, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 296, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 325, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 396, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 397, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 398, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 413, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 414, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 415, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 422, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 423, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 424, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 443, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 444, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 445, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 455, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 456, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 457, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 475, Character: 42},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 531, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 532, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 533, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 534, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 542, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 543, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 544, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 545, Character: 10},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 571, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 572, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 573, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 643, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 644, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 654, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 655, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 656, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 657, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 680, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/stream.go", Line: 681, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 203, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 204, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 287, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 288, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 318, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 319, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 345, Character: 67},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 356, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 357, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 385, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/transport.go", Line: 386, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/util.go", Line: 92, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/util.go", Line: 100, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/util.go", Line: 101, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/rafthttp/util.go", Line: 102, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/db.go", Line: 67, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/db.go", Line: 69, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/db.go", Line: 90, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 92, Character: 47},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 95, Character: 56},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 127, Character: 45},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 131, Character: 55},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 131, Character: 82},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 135, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 135, Character: 76},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 147, Character: 45},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 154, Character: 53},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 162, Character: 54},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 169, Character: 53},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 178, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 189, Character: 54},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 229, Character: 61},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/snap/snapshotter.go", Line: 242, Character: 66},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 211, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 250, Character: 33},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 274, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 313, Character: 38},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 331, Character: 37},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 358, Character: 38},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 378, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 383, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 437, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 438, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 448, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 449, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 549, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2auth/auth.go", Line: 560, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 115, Character: 45},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 235, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 236, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 238, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 256, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 257, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 259, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 298, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 299, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 337, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 338, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 343, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 344, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 353, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 362, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 364, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 372, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 373, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2discovery/discovery.go", Line: 379, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client.go", Line: 240, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client.go", Line: 267, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client.go", Line: 297, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client.go", Line: 574, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client.go", Line: 581, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 56, Character: 36},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 69, Character: 37},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 74, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 75, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 114, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 115, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 116, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 162, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 163, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 180, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 181, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 191, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 249, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 295, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 342, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 419, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 483, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 546, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/client_auth.go", Line: 584, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/http.go", Line: 44, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/http.go", Line: 45, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/http.go", Line: 74, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/http.go", Line: 75, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v2http/http.go", Line: 76, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v3rpc/interceptor.go", Line: 168, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v3rpc/interceptor.go", Line: 169, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/api/v3rpc/interceptor.go", Line: 174, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply.go", Line: 680, Character: 30},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply.go", Line: 680, Character: 69},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply.go", Line: 687, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply.go", Line: 703, Character: 33},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply.go", Line: 703, Character: 72},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply.go", Line: 706, Character: 51},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/apply_v2.go", Line: 82, Character: 42},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/backend.go", Line: 76, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/backend.go", Line: 82, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 77, Character: 50},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 85, Character: 59},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 92, Character: 56},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 101, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 102, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 153, Character: 40},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 177, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 178, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 186, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 187, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 188, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 231, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 232, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 240, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 241, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 242, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 249, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 250, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 251, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 277, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 278, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 289, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 290, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 299, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/cluster_util.go", Line: 300, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 47, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 61, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 65, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 89, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 93, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 100, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 104, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 117, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 131, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 214, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 225, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 238, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 290, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/corrupt.go", Line: 292, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 82, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 95, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 97, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 109, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 111, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 113, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 118, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/quota.go", Line: 120, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 347, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 423, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 424, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 469, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 470, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 553, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 554, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/raft.go", Line: 619, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 298, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 300, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 313, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 412, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 427, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 437, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 439, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 628, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 630, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 632, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 667, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 669, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 671, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 731, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 732, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 733, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 734, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 740, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 741, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 742, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 810, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 811, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 993, Character: 12},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1205, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1235, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1236, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1237, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1252, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1253, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1254, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1266, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1267, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1275, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1276, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1312, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1408, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1409, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1418, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1419, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1537, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1538, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1596, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1597, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1614, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1615, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1728, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1729, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1730, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1782, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1783, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1799, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1800, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1801, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1809, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1810, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1851, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1852, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1853, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1854, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1862, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1863, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1872, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1873, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1874, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1887, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1888, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1890, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 1954, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2036, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2071, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2072, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2110, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2111, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2272, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2277, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2278, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/server.go", Line: 2290, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/snapshot_merge.go", Line: 68, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/snapshot_merge.go", Line: 73, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/util.go", Line: 122, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/util.go", Line: 123, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/util.go", Line: 161, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/util.go", Line: 162, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/util.go", Line: 163, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "etcdserver/v3_server.go", Line: 431, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 40, Character: 38},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 93, Character: 42},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 109, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 111, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 112, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 134, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 140, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 166, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 167, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 193, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 194, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 218, Character: 54},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 228, Character: 54},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 242, Character: 52},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 252, Character: 52},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 273, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 274, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 293, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 294, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 333, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 334, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 335, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 368, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 369, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 370, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 385, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 386, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 411, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 413, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 426, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 427, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 452, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 454, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 475, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 571, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 572, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 655, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 665, Character: 51},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/handler.go", Line: 693, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/server.go", Line: 100, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/server.go", Line: 101, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/server.go", Line: 107, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/server.go", Line: 113, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/server.go", Line: 122, Character: 41},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/agent/server.go", Line: 124, Character: 40},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 278, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 280, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 312, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 314, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 315, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 316, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 320, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 338, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 340, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 358, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 360, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 361, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 362, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/rpcpb/member.go", Line: 366, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case.go", Line: 253, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case.go", Line: 296, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_delay.go", Line: 35, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_network_delay.go", Line: 37, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_no_fail.go", Line: 62, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_no_fail.go", Line: 70, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 44, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 61, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 62, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 68, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 69, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 93, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 94, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 95, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 102, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 103, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 104, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 120, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 138, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 139, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 152, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove.go", Line: 169, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 46, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 53, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 60, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 61, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 63, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 64, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 65, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 69, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 90, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 112, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 117, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 129, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 134, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 158, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 164, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 189, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 197, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 215, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/case_sigquit_remove_quorum.go", Line: 221, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_kv_hash.go", Line: 64, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_kv_hash.go", Line: 65, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 120, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 129, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 138, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 157, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 166, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 214, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 215, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 230, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/checker_lease_expire.go", Line: 231, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 92, Character: 32},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 98, Character: 37},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 139, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 144, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 297, Character: 45},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 301, Character: 45},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 350, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 356, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 367, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 459, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 460, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 471, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 472, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 474, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 480, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 481, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 527, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 541, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 554, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 577, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 584, Character: 48},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 602, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 610, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 678, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 689, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 696, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 708, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 735, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster.go", Line: 742, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_read_config.go", Line: 35, Character: 42},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 38, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 142, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 158, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 171, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 185, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 197, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 209, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 226, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 244, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/cluster_run.go", Line: 254, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser.go", Line: 57, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser.go", Line: 59, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 106, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 107, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 187, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 188, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 189, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 190, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 213, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_key.go", Line: 214, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 125, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 126, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 164, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 165, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 170, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 171, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 176, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 177, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 182, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 183, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 213, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 251, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 252, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 260, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 261, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 262, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 282, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 283, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 294, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 295, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 296, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 324, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 325, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 326, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 339, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 340, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 341, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 350, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 351, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 352, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 364, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 365, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 366, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 374, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 375, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 376, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 383, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 384, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 385, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 395, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 396, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 397, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 406, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 407, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 408, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 459, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 460, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 461, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 478, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_lease.go", Line: 479, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_runner.go", Line: 93, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "functional/tester/stresser_runner.go", Line: 104, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 794, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 797, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 951, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 954, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1052, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1055, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1061, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1064, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1086, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1089, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1111, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1114, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1124, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1127, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1137, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "integration/cluster.go", Line: 1140, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 156, Character: 51},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 248, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 385, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 387, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 389, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 418, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 437, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 440, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/backend.go", Line: 443, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/batch_tx.go", Line: 75, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/batch_tx.go", Line: 97, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/batch_tx.go", Line: 108, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/batch_tx.go", Line: 121, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/batch_tx.go", Line: 155, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/backend/batch_tx.go", Line: 162, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/key_index.go", Line: 122, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/key_index.go", Line: 140, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/key_index.go", Line: 163, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/key_index.go", Line: 207, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/kvstore.go", Line: 360, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/kvstore.go", Line: 361, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/kvstore.go", Line: 413, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/kvstore.go", Line: 428, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/kvstore.go", Line: 429, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "mvcc/kvstore_txn.go", Line: 268, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/fileutil/purge.go", Line: 72, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/fileutil/purge.go", Line: 76, Character: 26},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/flags/flag.go", Line: 76, Character: 54},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/flags/flag.go", Line: 86, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/flags/flag.go", Line: 92, Character: 53},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/flags/flag.go", Line: 114, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/flags/flag.go", Line: 115, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/flags/ignored.go", Line: 33, Character: 58},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 99, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 100, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 113, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 114, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 115, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 122, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 123, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 132, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/netutil/netutil.go", Line: 133, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/osutil/interrupt_unix.go", Line: 62, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 277, Character: 35},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 277, Character: 65},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 297, Character: 40},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 345, Character: 61},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 486, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 487, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 488, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 489, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 494, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 495, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 496, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 497, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 614, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 615, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 616, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 621, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 622, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 623, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 642, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 643, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 660, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 661, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 679, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 680, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 698, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 699, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 712, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 713, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 738, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 739, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 752, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 753, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 778, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 779, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 792, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 793, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 830, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 831, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 842, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 843, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 853, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 854, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 865, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 866, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 874, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 875, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 883, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 884, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 892, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 893, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 901, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 902, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 913, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 914, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 932, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 933, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 944, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 945, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 963, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 964, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/proxy/server.go", Line: 993, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/traceutil/trace.go", Line: 202, Character: 23},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 198, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 206, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 218, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 227, Character: 43},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 311, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 312, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 320, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 321, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 334, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 335, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 343, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "pkg/transport/listener.go", Line: 344, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/grpcproxy/cluster.go", Line: 86, Character: 46},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/grpcproxy/register.go", Line: 77, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/director.go", Line: 150, Character: 48},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/director.go", Line: 157, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/director.go", Line: 173, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/reverse.go", Line: 82, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/reverse.go", Line: 109, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/reverse.go", Line: 130, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/reverse.go", Line: 158, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/httpproxy/reverse.go", Line: 177, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/tcpproxy/userspace.go", Line: 177, Character: 46},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/tcpproxy/userspace.go", Line: 209, Character: 94},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "proxy/tcpproxy/userspace.go", Line: 213, Character: 39},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "tools/etcd-dump-metrics/main.go", Line: 182, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 39, Character: 26},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 62, Character: 27},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 68, Character: 48},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 74, Character: 39},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 79, Character: 34},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 79, Character: 74},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 84, Character: 38},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 90, Character: 35},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 95, Character: 27},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/repair.go", Line: 99, Character: 35},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/util.go", Line: 45, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/util.go", Line: 61, Character: 49},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/util.go", Line: 91, Character: 9},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 117, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 118, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 129, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 137, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 145, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 176, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 177, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 195, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 196, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 205, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 206, Character: 8},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 218, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 219, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 242, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 243, Character: 7},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 275, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 276, Character: 6},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 699, Character: 44},
			{Repo: "github.com/sourcegraph-testing/etcd", Rev: "ad7848014a051dbe3fcd6a4cff2c7befdd16d5a8", Path: "wal/wal.go", Line: 890, Character: 45},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "bindinfo/handle.go", Line: 600, Character: 52},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "bindinfo/handle.go", Line: 617, Character: 57},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "bindinfo/handle.go", Line: 634, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "cmd/benchdb/main.go", Line: 148, Character: 38},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "cmd/benchdb/main.go", Line: 154, Character: 38},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "cmd/explaintest/main.go", Line: 674, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "cmd/explaintest/main.go", Line: 676, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "cmd/importer/stats.go", Line: 79, Character: 47},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "cmd/importer/stats.go", Line: 79, Character: 75},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/callback_test.go", Line: 51, Character: 35},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/callback_test.go", Line: 65, Character: 32},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/column.go", Line: 187, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/column.go", Line: 316, Character: 60},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/db_change_test.go", Line: 1235, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/db_change_test.go", Line: 1248, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 207, Character: 100},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 236, Character: 66},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 300, Character: 47},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 320, Character: 48},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 331, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 355, Character: 72},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 371, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 406, Character: 49},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 499, Character: 52},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl.go", Line: 499, Character: 85},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_test.go", Line: 81, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_test.go", Line: 98, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_test.go", Line: 98, Character: 92},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 189, Character: 77},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 189, Character: 120},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 252, Character: 87},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 281, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 306, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 358, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 398, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 567, Character: 56},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/ddl_worker.go", Line: 763, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/delete_range.go", Line: 97, Character: 98},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 461, Character: 57},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 514, Character: 86},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 527, Character: 91},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 940, Character: 104},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 950, Character: 91},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1127, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1127, Character: 167},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1137, Character: 85},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1148, Character: 93},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1287, Character: 53},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1287, Character: 97},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1295, Character: 89},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1377, Character: 63},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/index.go", Line: 1377, Character: 96},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/reorg.go", Line: 219, Character: 71},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/reorg.go", Line: 379, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/rollingback.go", Line: 228, Character: 72},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/rollingback.go", Line: 369, Character: 66},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/rollingback.go", Line: 371, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 148, Character: 62},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 148, Character: 86},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 302, Character: 66},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 390, Character: 116},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 390, Character: 151},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 397, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 484, Character: 91},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 501, Character: 92},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "ddl/util/syncer.go", Line: 505, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/domain.go", Line: 1179, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/domain.go", Line: 1180, Character: 27},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/infosync/info.go", Line: 247, Character: 84},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/infosync/info.go", Line: 253, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/infosync/info.go", Line: 253, Character: 44},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/infosync/info.go", Line: 578, Character: 49},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "domain/infosync/info.go", Line: 588, Character: 50},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/adapter.go", Line: 122, Character: 53},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/adapter.go", Line: 294, Character: 53},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/adapter.go", Line: 606, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/adapter.go", Line: 684, Character: 121},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/admin.go", Line: 312, Character: 49},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/admin.go", Line: 313, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/admin.go", Line: 399, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/admin.go", Line: 526, Character: 60},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/admin.go", Line: 527, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/analyze.go", Line: 170, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/analyze.go", Line: 330, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/analyze.go", Line: 330, Character: 107},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/ddl.go", Line: 308, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/ddl.go", Line: 309, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/distsql.go", Line: 650, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/distsql.go", Line: 808, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/distsql.go", Line: 957, Character: 63},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/distsql.go", Line: 959, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/distsql.go", Line: 960, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/executor.go", Line: 1506, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/index_lookup_join.go", Line: 331, Character: 57},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/index_lookup_join.go", Line: 447, Character: 57},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/insert.go", Line: 229, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/insert.go", Line: 231, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/prepared.go", Line: 253, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/projection.go", Line: 442, Character: 62},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/projection.go", Line: 442, Character: 105},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/replace.go", Line: 65, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/replace.go", Line: 66, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/set.go", Line: 206, Character: 96},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/set.go", Line: 206, Character: 122},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/set.go", Line: 210, Character: 97},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/set.go", Line: 210, Character: 123},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/slow_query.go", Line: 534, Character: 65},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/split.go", Line: 101, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/split.go", Line: 102, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/split.go", Line: 373, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/split.go", Line: 410, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/split.go", Line: 415, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "executor/split.go", Line: 416, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/builtin_string.go", Line: 2370, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/builtin_string.go", Line: 2371, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/builtin_string_vec.go", Line: 2258, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/builtin_string_vec.go", Line: 2259, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/constant_fold.go", Line: 50, Character: 63},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/constant_fold.go", Line: 198, Character: 63},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/constant_fold.go", Line: 216, Character: 64},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/expr_to_pb.go", Line: 83, Character: 69},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/expr_to_pb.go", Line: 182, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/expr_to_pb.go", Line: 184, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/expr_to_pb.go", Line: 198, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "expression/util.go", Line: 836, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "owner/manager.go", Line: 169, Character: 64},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "owner/manager.go", Line: 223, Character: 49},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "owner/manager.go", Line: 223, Character: 102},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "owner/manager.go", Line: 315, Character: 46},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "planner/core/exhaust_physical_plans.go", Line: 761, Character: 121},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "planner/core/exhaust_physical_plans.go", Line: 884, Character: 121},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "planner/core/exhaust_physical_plans.go", Line: 899, Character: 121},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "planner/core/planbuilder.go", Line: 1173, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "planner/core/planbuilder.go", Line: 1173, Character: 83},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "planner/core/stats.go", Line: 513, Character: 87},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 168, Character: 66},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 179, Character: 68},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 193, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 228, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 273, Character: 69},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 282, Character: 69},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "plugin/plugin.go", Line: 375, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/cache.go", Line: 624, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/cache.go", Line: 624, Character: 42},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/cache.go", Line: 763, Character: 56},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 111, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 111, Character: 33},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 116, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 135, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 135, Character: 33},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 160, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 160, Character: 33},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 171, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 171, Character: 34},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 181, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 181, Character: 33},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 188, Character: 80},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 231, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 231, Character: 38},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 241, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 241, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 247, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 247, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 259, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 259, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 265, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 265, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 269, Character: 63},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 269, Character: 94},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 270, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 270, Character: 52},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 289, Character: 65},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 289, Character: 96},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 290, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 290, Character: 55},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 299, Character: 66},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 299, Character: 97},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 300, Character: 10},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 300, Character: 56},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 308, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "privilege/privileges/privileges.go", Line: 308, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 663, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 664, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 730, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 731, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 732, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 734, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 735, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/conn.go", Line: 1399, Character: 108},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/http_handler.go", Line: 216, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/http_handler.go", Line: 1720, Character: 121},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/http_status.go", Line: 80, Character: 62},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 177, Character: 59},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 177, Character: 93},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 181, Character: 61},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 181, Character: 95},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 199, Character: 74},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 234, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 237, Character: 55},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 237, Character: 89},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 244, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 257, Character: 91},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 387, Character: 48},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 422, Character: 62},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "server/server.go", Line: 627, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 464, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 466, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 474, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 490, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 664, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 701, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 710, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 716, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 718, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1045, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1099, Character: 68},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1147, Character: 70},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1168, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1194, Character: 68},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1225, Character: 71},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 1700, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 2210, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 2216, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 2217, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 2235, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 2236, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/session.go", Line: 2237, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/tidb.go", Line: 71, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/tidb.go", Line: 160, Character: 43},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/tidb.go", Line: 171, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "session/txn.go", Line: 243, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/binloginfo/binloginfo.go", Line: 253, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 655, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 656, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 657, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 658, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 659, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 660, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 661, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "sessionctx/stmtctx/stmtctx.go", Line: 662, Character: 29},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/handle.go", Line: 189, Character: 72},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/handle.go", Line: 408, Character: 122},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/handle.go", Line: 496, Character: 121},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 721, Character: 64},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 721, Character: 88},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 728, Character: 70},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 742, Character: 62},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 797, Character: 40},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 797, Character: 78},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 805, Character: 40},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 805, Character: 78},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 825, Character: 40},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 826, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 827, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 832, Character: 41},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 833, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 834, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 839, Character: 41},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 840, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 883, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/handle/update.go", Line: 883, Character: 76},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/histogram.go", Line: 1052, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "statistics/histogram.go", Line: 1052, Character: 51},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/helper/helper.go", Line: 628, Character: 47},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/store.go", Line: 67, Character: 43},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/2pc.go", Line: 539, Character: 65},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/batch_coprocessor.go", Line: 336, Character: 56},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/batch_coprocessor.go", Line: 386, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 224, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 233, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 297, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 305, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 331, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 498, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 590, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 590, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 605, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 605, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/client_batch.go", Line: 628, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/coprocessor.go", Line: 956, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 163, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 190, Character: 52},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 233, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 248, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 255, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 262, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 397, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 409, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 415, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 423, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 443, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 494, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 509, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 517, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 535, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 545, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 556, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 564, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 573, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 583, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 592, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 615, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 624, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 636, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 644, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 667, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 676, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 688, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 696, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 708, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 896, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 916, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 932, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 942, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 956, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 963, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1044, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1068, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1139, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1148, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1178, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1188, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1217, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1247, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1256, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1420, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1426, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1459, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1460, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1510, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1526, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1536, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1561, Character: 56},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1588, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1700, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1705, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1706, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1719, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/gcworker/gc_worker.go", Line: 1720, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/lock_resolver.go", Line: 589, Character: 56},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 101, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 108, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 150, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 164, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 204, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 214, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/range_task.go", Line: 281, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 352, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 407, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 513, Character: 71},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1366, Character: 39},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1451, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1496, Character: 73},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1502, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1509, Character: 78},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1513, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_cache.go", Line: 1513, Character: 95},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_request.go", Line: 331, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_request.go", Line: 332, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/region_request.go", Line: 370, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/split_region.go", Line: 257, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "store/tikv/split_region.go", Line: 258, Character: 9},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "table/tables/partition.go", Line: 193, Character: 68},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "table/tables/partition.go", Line: 223, Character: 67},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "table/tables/partition.go", Line: 397, Character: 65},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "table/tables/tables.go", Line: 869, Character: 127},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "table/tables/tables.go", Line: 876, Character: 126},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "tidb-server/main.go", Line: 231, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "tidb-server/main.go", Line: 343, Character: 46},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "tidb-server/main.go", Line: 343, Character: 79},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "tidb-server/main.go", Line: 357, Character: 69},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "tidb-server/main.go", Line: 379, Character: 42},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "tidb-server/main.go", Line: 671, Character: 46},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "types/mydecimal.go", Line: 2296, Character: 53},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "types/mydecimal.go", Line: 2306, Character: 53},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/admin/admin.go", Line: 324, Character: 7},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/collate/collate.go", Line: 131, Character: 8},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 189, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 192, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 195, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 198, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 201, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 204, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 207, Character: 30},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 212, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 215, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 218, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 222, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 226, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 231, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/execdetails/execdetails.go", Line: 234, Character: 31},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 104, Character: 35},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 127, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 133, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 136, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 141, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 145, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 149, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/expensivequery/expensivequery.go", Line: 160, Character: 35},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log.go", Line: 365, Character: 58},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log_test.go", Line: 183, Character: 47},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log_test.go", Line: 184, Character: 45},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log_test.go", Line: 232, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log_test.go", Line: 233, Character: 34},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log_test.go", Line: 234, Character: 34},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/logutil/log_test.go", Line: 235, Character: 36},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/printer/printer.go", Line: 41, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/printer/printer.go", Line: 42, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/printer/printer.go", Line: 43, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/printer/printer.go", Line: 44, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/printer/printer.go", Line: 45, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/printer/printer.go", Line: 48, Character: 6},
			{Repo: "github.com/sourcegraph-testing/tidb", Rev: "9aab49176993f9dc0ed2fcb9ef7e5125518e8b98", Path: "util/stmtsummary/statement_summary.go", Line: 743, Character: 74},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "example_test.go", Line: 57, Character: 6},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "example_test.go", Line: 264, Character: 7},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "example_test.go", Line: 265, Character: 7},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field.go", Line: 215, Character: 5},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field.go", Line: 225, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field.go", Line: 362, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field.go", Line: 462, Character: 9},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field_test.go", Line: 106, Character: 71},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field_test.go", Line: 146, Character: 32},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field_test.go", Line: 210, Character: 43},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "field_test.go", Line: 212, Character: 42},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_bench_test.go", Line: 104, Character: 23},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 86, Character: 42},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 91, Character: 57},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 103, Character: 14},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 104, Character: 14},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 108, Character: 37},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 109, Character: 37},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 420, Character: 23},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 424, Character: 19},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "logger_test.go", Line: 439, Character: 22},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 76, Character: 37},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 106, Character: 21},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 106, Character: 63},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 118, Character: 59},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 173, Character: 27},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 205, Character: 27},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "sugar_test.go", Line: 241, Character: 27},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/field_test.go", Line: 176, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/field_test.go", Line: 177, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/field_test.go", Line: 181, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/field_test.go", Line: 182, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/field_test.go", Line: 186, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/field_test.go", Line: 187, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zapcore/json_encoder_test.go", Line: 85, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/logger_test.go", Line: 86, Character: 66},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/logged_entry_test.go", Line: 45, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/logged_entry_test.go", Line: 56, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/logged_entry_test.go", Line: 57, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/logged_entry_test.go", Line: 66, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/logged_entry_test.go", Line: 68, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/observer_test.go", Line: 125, Character: 32},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/observer_test.go", Line: 129, Character: 32},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "2aa9fa25da83bdfff756c36a91442edc9a84576c", Path: "zaptest/observer/observer_test.go", Line: 170, Character: 34},
		},
	},
	{
		Definition: Location{
			Repo:      "github.com/sourcegraph-testing/zap",
			Rev:       "a6015e13fab9b744d96085308ce4e8f11bad1996",
			Path:      "field.go",
			Line:      119,
			Character: 5,
		},
		References: []Location{
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 53, Character: 74},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 58, Character: 40},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 61, Character: 41},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 63, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 66, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 67, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 89, Character: 42},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 93, Character: 45},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 119, Character: 32},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 129, Character: 17},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 131, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "client.go", Line: 132, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 189, Character: 41},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 189, Character: 71},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 196, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 197, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 205, Character: 37},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 205, Character: 67},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 213, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 214, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 233, Character: 10},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 234, Character: 10},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 242, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 243, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 247, Character: 32},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 247, Character: 62},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 254, Character: 34},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 254, Character: 64},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/command.go", Line: 276, Character: 42},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 45, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 46, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 69, Character: 33},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 89, Character: 30},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 97, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 98, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 104, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 105, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 114, Character: 7},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 115, Character: 7},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 136, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "command/transactions.go", Line: 137, Character: 8},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "db/expire.go", Line: 167, Character: 21},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "db/kv.go", Line: 341, Character: 9},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "db/string.go", Line: 80, Character: 42},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "db/ztransfer.go", Line: 67, Character: 19},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 32, Character: 41},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 39, Character: 40},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 42, Character: 50},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 46, Character: 47},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 52, Character: 48},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 57, Character: 55},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 61, Character: 50},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 69, Character: 52},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "metrics/server.go", Line: 72, Character: 41},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 27, Character: 40},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 33, Character: 45},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 43, Character: 38},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 44, Character: 41},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 49, Character: 43},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 69, Character: 38},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "titan.go", Line: 75, Character: 42},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "tools/expire/main.go", Line: 30, Character: 26},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "tools/expire/main.go", Line: 75, Character: 27},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "tools/expire/main.go", Line: 102, Character: 6},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "tools/expire/main.go", Line: 124, Character: 6},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "tools/expire/main.go", Line: 139, Character: 35},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "util.go", Line: 14, Character: 33},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "util.go", Line: 15, Character: 33},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "util.go", Line: 16, Character: 33},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "util.go", Line: 17, Character: 33},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "util.go", Line: 18, Character: 33},
			{Repo: "github.com/sourcegraph-testing/titan", Rev: "f8307e394c512b4263fc0cd67ccf9fd46f1ad9a5", Path: "util.go", Line: 19, Character: 33},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "example_test.go", Line: 57, Character: 6},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "example_test.go", Line: 264, Character: 7},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "example_test.go", Line: 265, Character: 7},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "field.go", Line: 119, Character: 5},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "field.go", Line: 194, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "field.go", Line: 265, Character: 9},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "field_test.go", Line: 83, Character: 71},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "field_test.go", Line: 122, Character: 32},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_bench_test.go", Line: 104, Character: 23},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 86, Character: 42},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 91, Character: 57},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 103, Character: 14},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 104, Character: 14},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 108, Character: 37},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 109, Character: 37},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 405, Character: 23},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 409, Character: 19},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "logger_test.go", Line: 424, Character: 22},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 76, Character: 37},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 106, Character: 21},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 106, Character: 63},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 118, Character: 59},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 177, Character: 27},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 209, Character: 27},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "sugar_test.go", Line: 245, Character: 27},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/field_test.go", Line: 176, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/field_test.go", Line: 177, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/field_test.go", Line: 181, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/field_test.go", Line: 182, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/field_test.go", Line: 186, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/field_test.go", Line: 187, Character: 13},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zapcore/json_encoder_test.go", Line: 83, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/logger_test.go", Line: 86, Character: 66},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/logged_entry_test.go", Line: 45, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/logged_entry_test.go", Line: 56, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/logged_entry_test.go", Line: 57, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/logged_entry_test.go", Line: 66, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/logged_entry_test.go", Line: 68, Character: 8},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/observer_test.go", Line: 125, Character: 32},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/observer_test.go", Line: 129, Character: 32},
			{Repo: "github.com/sourcegraph-testing/zap", Rev: "a6015e13fab9b744d96085308ce4e8f11bad1996", Path: "zaptest/observer/observer_test.go", Line: 170, Character: 34},
		},
	},
}
